# Tilgungsplan

proc gui {} {
  wm title . "Tilgungsplan"

  # Konstruktion Menubar
  menu .menubar
  .menubar add cascade -label File -underline 0 -menu .menubar.file
  # Men File
  menu .menubar.file -tearoff 0
  .menubar.file add command -label Exit -command {exit}
  . config -menu .menubar

  # Eingabefelder
  label .lhyp -text "Hypothek:"
  entry .ehyp -textvariable hypothek
  label .lzins -text "Zinssatz:"
  entry .ezins -textvariable zinssatz100
  label .ltilg -text "Tilgungssatz:"
  entry .etilg -textvariable tilgsatz100
  label .llauf -text "Laufzeit:"
  entry .elauf -textvariable laufzeit
  button .bcomp -text Berechnung -command {compute}
  grid .lhyp -row 0 -column 0
  grid .ehyp -row 0 -column 1
  grid .lzins -row 1 -column 0
  grid .ezins -row 1 -column 1
  grid .ltilg -row 2 -column 0
  grid .etilg -row 2 -column 1
  grid .llauf -row 3 -column 0
  grid .elauf -row 3 -column 1
  grid .bcomp -row 3 -column 3
  # Ausgabefeld
  text .txt -font fixed -yscrollcommand {.sb set} -width 50
  scrollbar .sb -command {.txt yview}
  grid .txt -row 4 -column 0 -columnspan 2
  grid .sb -row 4 -column 2 -sticky ns
  # Graphische Ausgabe
  canvas .c -width 310 -height 310 -background white
  grid .c -row 4 -column 3 -sticky ns
}

proc convert_input {} {
  # Globale Variable spezif.
  global hypothek zinssatz tilgsatz laufzeit zinssatz100 tilgsatz100

  # Evt. Komma durch Dez.-Punkt ersetzen
  if {$hypothek == {}} {set hypothek 0}
  regsub {,} $hypothek {.} hypothek
  if {$zinssatz100 == {}} {set zinssatz100 0}
  regsub {,} $zinssatz100 {.} zinssatz100
  if {$tilgsatz100 == {}} {set tilgsatz100 0}
  regsub {,} $tilgsatz100 {.} tilgsatz100
  if {$laufzeit == {}} {set laufzeit 0}
  set zinssatz [expr $zinssatz100*0.01]
  set tilgsatz [expr $tilgsatz100*0.01]
}

proc compute {} {
  # Globale Variable spezif.
  global hypothek zinssatz tilgsatz laufzeit zins tilg

  # Umrechnung der Eingaben
  convert_input
  # Lschen der Ausgabe
  .txt delete 1.0 end
  # Ausgangswert erhalten, deshalb mit Kopie weiter
  set hyp $hypothek
  set belastung [expr $hyp*($zinssatz+$tilgsatz)]
  .txt insert end "Hypothekentilgungsplan\n"
  .txt insert end "======================\n\n"
  .txt insert end [format \
    "Hypothek: %12.2f DM, Zinssatz: %5.2f %%\n" \
    $hypothek [expr $zinssatz*100]]
  .txt insert end [format \
    "Belastung: %11.2f DM, Tilgsatz: %5.2f %%\n" \
    $belastung [expr $tilgsatz*100]]
  .txt insert end [format \
    "Laufzeit:  %11d Jahre\n\n" $laufzeit]

  # Tabellenkopf
  .txt insert end "Jahr   Zinsen  Tilgung   Restschuld\n"
  .txt insert end "===================================\n"
  for {set i 1} {$i <= $laufzeit} {incr i} {
    set zinsen [expr $hyp*$zinssatz]
    set zins($i) $zinsen
    set tilgung [expr $belastung-$zinsen]
    set tilg($i) $tilgung
    set hyp [expr $hyp-$tilgung]
    .txt insert end [format \
      "%4d %8.2f %8.2f %12.2f\n" $i $zinsen $tilgung $hyp]
  }
  graph
}

proc graph {} {
  global laufzeit zins tilg

  # max ist belastung in vollen 1000ern
  set max [expr $zins(1)+$tilg(1)]
  set max [expr ceil($max*0.001)]
  .c delete all
  .c create line 55 255 308 255
  .c create line 55 2 55 255
  set dx [expr 250/$laufzeit]
  set dy [expr 250/$max]
  for {set i 1} {$i <= $laufzeit} {incr i} {
    set x [expr 55+$i*$dx]
    .c create line $x 252 $x 258
    .c create text $x 270 -text $i
  }
  for {set i 1} {$i <= $max} {incr i} {
    set y [expr 255-$i*$dy]
    .c create line 52 $y 58 $y
    .c create text 20 $y -text [expr $i*1000]
  }
  set dy [expr 0.001*$dy]
  # Belastungsgerade zeichnen
  set y [expr 255-($zins(1)+$tilg(1))*$dy]
  .c create line 55 $y 305 $y -fill blue
  # Jahr 1 berechnen
  set x [expr 55+$dx]
  set y [expr 255-$zins(1)*$dy]
  for {set i 2} {$i <= $laufzeit} {incr i} {
    set x0 $x
    set x [expr 55+$i*$dx]
    set y0 $y
    set y [expr 255-$zins($i)*$dy]
    .c create line $x0 $y0 $x $y -fill red
  }  
}

# Hauptprogramm
gui
